import numpy as np

def fix_coords(dataset):
    """ Function that sets longitudes in the -180,180 range and sorts 
        latitudes.
        
        dataset: xarray object with 2D coords dims (lat,lon) or (latitude,longitude)
        return: arranged dataset
    """  
    # Changes the name of the lon lat coords if necessary
    try:
        dataset = dataset.rename({'longitude':'lon','latitude':'lat'})
    except:
        #log = "no need to rename lon lat dims"
        pass
    try:
        dataset = dataset.rename({'X':'lon','Y':'lat'})
    except:
        #log = "no need to rename lon lat dims"
        pass
    # Converts longitudes to 0 360 domain
    dataset = dataset.assign_coords(lon=(((dataset.lon + 180) % 360) - 180))
    dataset = dataset.sortby('lon')
    dataset = dataset.sortby('lat')
    # Round times to start of the day
    try:
        dataset = dataset.assign_coords(time=dataset.time.dt.floor("D"))
    except:
        #log = "no need to reset hour un leadtimes"
        pass
    return dataset
    
def cut_domain(dataset,domain):
    """ Function to cut domain.

    dataset: xarray object with (lat,lon) coords, previously fixed w fix_coords
    domain: list [lonmin, lonmax, latmin, latmax]
    """
    # Cuts dataset to the provided domain
    return dataset.sel(dict(lon=slice(domain[0],domain[1]), lat=slice(domain[2],domain[3])))

def convert_prlr(dataset,units):
    """ Function that converts the precipitation fields to mm.
        
        dataset: xarray object containing a prlr field
        units: units of the provided dataset ("m/h" or "m/s")
        return: dataset with the prlr fields converted to mm
    """
    # Converts the prlr fields
    if units == "m/h":
        dataset['prlr'] = dataset['prlr']*24*31*1000 # m/h to mm
    elif units == "m/s":
        dataset['prlr'] = dataset['prlr']*(3600*24*31)*1000 # m/s to mm
    else:
        raise ValueError("Units don't match m/s or m/h")
    return(dataset)

def scale(da):
    mean=da.mean(skipna=True)
    sd=(da-mean).std(skipna=True)
    #sd=np.asarray(da).std()
    return (da-mean)/sd

def merge_features(dsf, varnames):
    tmp = [dsf[v].stack(pos=("lon","lat")) for v in varnames]
    return np.concatenate(tmp, axis=1)

def subset_stacked_da_onevar(da, var, lonmin, lonmax, latmin, latmax):
    """ Subset region of a stacked dataarray for just one variable
    """
    mask = (
        (
            (da.coords["lon"] >= lonmin)
            & (da.coords["lon"] <= lonmax)
            & (da.coords["lat"] >= latmin)
            & (da.coords["lat"] <= latmax)
        )
        & (da.coords["variable"] == var)
    ) | (da.coords["variable"] != var)
    return da.where(mask, drop=True)

def subset_stacked_da_allvar(da, vars_dict):
    """ Subset region of a stacked dataarray for a list of variables
        Variables not listed are excluded
    """ 
    masks = [
        (
            (da.coords["lon"] >= region[0])
            & (da.coords["lon"] <= region[1])
            & (da.coords["lat"] >= region[2])
            & (da.coords["lat"] <= region[3])
        ) 
        & (da.coords["variable"] == var)
        for (var, region) in vars_dict.items()
    ]
    mask = False
    for m in masks:
        mask = mask | m
    return da.where(mask, drop=True)

# default multioutput mean_squared_error returns the rmse across time for each pixel
# whereas this version returns the image rmse for each sample (time instance)
def rms_error(y_true, y_pred):
    sse = (y_true - y_pred)**2
    rmse = np.sqrt(sse.mean(axis=1))
    return rmse.mean()

