from sklearn.base import BaseEstimator, TransformerMixin
import numpy as np

class VarSelector(BaseEstimator, TransformerMixin):
    """
    Subset variables from a stacked xr.DataSet
    Class to use in the pipline
    -----
    attributes 
    fit : identify columns - in the training set
    transform : subset to only use those columns
    """
    
    def __init__(self, varlist=None):
        self.varlist = varlist
    
    def fit(self, X, y=None):
        if self.varlist is not None:
            self.columns_to_keep_ = X.coords['variable'].isin(self.varlist)
        return self
    
    def transform(self, X, y=None, **kwargs):
        return X[:,self.columns_to_keep_]


class VarScaler(BaseEstimator, TransformerMixin):
    """
    Scale variables from a stacked xr.DataSet
    computes a single scaling per variable
    -----
    attributes 
    fit : find mean and variance for each variable - in the training set
    transform : standardize each variable field
    """
    
    def __init__(self):
        pass
    
    def fit(self, X, y=None):
        tmp = X.unstack().to_dataset("variable")
        self.mean_ = tmp.mean(skipna=True)
        self.std_ = (tmp-self.mean_).std(skipna=True)
        return self
    
    def transform(self, X, y=None, **kwargs):
        tmp = X.unstack().to_dataset("variable")
        tmp -= self.mean_
        tmp /= self.std_
        return tmp.to_array().stack(feature=("variable","lon","lat"))


class VarSubset(BaseEstimator, TransformerMixin):
    """ 
    Subset the region of a stacked dataarray for just one variable
    -----
    attributes 
    fit : produce a mask with the var and the region to cut
    transform : use the fitted mask to cut the selected var to the selected region
    """
    def __init__(self, var=None, region=None):
        self.var=var
        self.region=region
    
    def fit(self, X, y=None):
        if self.var is not None:
            self.lonmin_ = self.region[0]
            self.lonmax_ = self.region[1]
            self.latmin_ = self.region[2]
            self.latmax_ = self.region[3]
            self.mask_ = (
                (
                    (X.coords["lon"] >= self.lonmin_)
                    & (X.coords["lon"] <= self.lonmax_)
                    & (X.coords["lat"] >= self.latmin_)
                    & (X.coords["lat"] <= self.latmax_)
                )
                & (X.coords["variable"] == self.var)
            ) | (X.coords["variable"] != self.var)
        return self
    
    def transform(self, X, y=None, **kwargs):
        return X.where(self.mask_, drop=True)


class VarWeight(BaseEstimator, TransformerMixin):
    """ 
    Weight the variable to take more or less importance in distance calculations
    -----
    attributes 
    fit : find the columns that relate to the selected var. Then divide the weight by number of points in the variable.
    transform : multiply each column of the selected var by the weight
    """
    def __init__(self, var=None, weight=None):
        self.var=var
        self.weight=weight
    
    def fit(self, X, y=None):
        if self.var is not None:
            self.mask_ = X.coords["variable"] == self.var
            self.npoints_ = sum(self.mask_.values)
            if self.npoints_ > 0:
                self.weight_ = self.weight / sum(self.mask_.values)
            else:
                self.weight_ = self.weight
        else:
            self.npoints_ = 0
        return self
    
    def transform(self, X, y=None, **kwargs):
        if self.npoints_ > 0:
            tmp = X.copy()
            tmp.loc[dict(feature=self.mask_)] *= self.weight_
            return tmp
        else:
            return X


class AddDayOfYear(BaseEstimator, TransformerMixin):
    """ 
    Add two new features (columns) for day of year. 
    To account for the cyclical nature (i.e. day 1 is close to day 365), day of year is normalized to [0,2pi] and then sinus and cosinus are computed.
    A weight can be specified so that these columns have more or less effect on distance calculation.
    -----
    attributes 
    fit : do nothing
    transform : compute and add the two new columns. This breaks the Xarray structure of X and produces a numpy array
    """
    def __init__(self, weight=None):
        self.weight=weight
    
    def fit(self, X, y=None):
        if self.weight is not None:
            self.weight_ = self.weight
        else:
            self.weight_ = 1
        return self
    
    def transform(self, X, y=None, **kwargs):
        sinDOY = np.sin(2 * np.pi * (X.time.dt.dayofyear - 1) / 364) * self.weight_
        cosDOY = np.cos(2 * np.pi * (X.time.dt.dayofyear - 1) / 364) * self.weight_
        return np.hstack((X.values, np.column_stack((sinDOY, cosDOY))))

