# module load R/3.6.1-foss-2015a-bare
# module load CDO/1.9.8-foss-2015a
# module load HDF5/1.8.14-foss-2015a

library(startR)
library(lubridate)
library(s2dv)
library(easyVerification)
library(multiApply)

###################################
# Training and test period configuration
###################################
sdates <- seq(ymd("19791201"), ymd("20181101"), by='1 month')

###################################
# Dataset configuration
###################################
datasets <- list(
	mswep	= '/esarchive/recon/gloh2o/mswep-v2.6/monthly_mean/$var$_f3h/$var$_$sdate$.nc',
	mescan	= '/esarchive/recon/ecmwf/uerra_mescan/monthly_mean/$var$_f6h/$var$_$sdate$.nc'
)

config <- list()
#config$var <- "prlr"
#config$dataset <- datasets$mswep
config$var <- "tas"
config$dataset <- datasets$mescan
config$sdates <- sdates
config$lonmin <- -30
config$lonmax <- 20
config$latmin <- 30
config$latmax <- 50
	
LoadObsData <- function(config) {
	tmp <- Start(
		sdate = format(config$sdates,"%Y%m"),
		time = 'all',
		longitude = values(list(config$lonmin, config$lonmax)),
		latitude = values(list(config$latmin, config$latmax)),
		longitude_reorder = CircularSort(-180, 180),
		#latitude_reorder = Sort(),
		longitude_var = "longitude",
		latitude_var = "latitude",
		var = config$var,
		dat = config$dataset,
		pattern_dims = 'dat',
		num_procs = 4, retrieve = T,
		synonims = list(longitude = c('lon', 'longitude'), latitude=c('lat', 'latitude'))
	)

	return(tmp)
}

data <- drop(LoadObsData(config))
nlons <- unname(dim(data)["longitude"])
nlats <- unname(dim(data)["latitude"])
nmonths <- unname(dim(data)["sdate"])
nyears <- nmonths / 12

lons <- as.vector(attributes(data)$Variables$common$lon)
lats <- as.vector(attributes(data)$Variables$common$lat)
files <- attributes(data)$Files

dim(data) <- c(month = 3, seas = 4, year = nyears , lon = nlons, lat = nlats)
var.hr <- MeanDims(data, "month")

# Interpolate to C3S grid (1x1deg) and back to the original grid for one to one comparisons
# Conservative produces a grid average when going from fine to coarse, but a nearest interpolation from coarse to fine!
var.1deg <- s2dverification::CDORemap(var.hr, lon = as.vector(lons), lat = as.vector(lats), method = "conservative", grid ="/esarchive/exp/ecmwf/system5c3s/monthly_mean/tas_f6h/tas_19990101.nc")
var.1deg.reint <- s2dverification::CDORemap(var.1deg$data_array, lon = var.1deg$lons, lat = var.1deg$lats, method = "conservative", grid = files[1])

var.lr <- var.1deg.reint$data_array

lons2 <- var.1deg.reint$lons
lats2 <- var.1deg.reint$lats
lons2 <- ifelse(lons2 >= 180, lons2 - 360, lons2)

#remove unwanted extra points
var.hr <- var.hr[,,lons > -17. & lons < 19, lats > 31 & lats < 49]
var.lr <- var.lr[,,lons2 > -17. & lons2 < 19, lats2 > 31 & lats2 < 49]
lons3 <- lons[lons > -17. & lons < 19]
lats3 <- lats[lats > 31 & lats < 49 ]

if(config$var == "prlr") {
	var.lr <- var.lr[,,,180:1]
}

# Compute correlation
C_cor <- stats:::C_cor
Corr <- Apply(list(var.hr, var.lr), "year", cor)$output1

# Compute terciles
t.hr <- Apply(var.hr, target_dims = "year", output_dims=c("year", "tercile"), fun= function(x) { convert2prob(as.vector(x), prob=1:2 / 3) } )$output1
t.lr <- Apply(var.lr, target_dims = "year", output_dims=c("year", "tercile"), fun= function(x) { convert2prob(as.vector(x), prob=1:2 / 3) } )$output1

# Percentage of correct terciles
pc <- Apply(list(t.lr, t.hr), target_dims=c("year", "tercile"), fun = function(x, y) { 1 - mean(rowSums(abs(x - y)) / 2)  })$output1

#Climo
climo <- MeanDims(var.hr, "year")

# Plots

if(config$var == "prlr") {
	PlotLayout(PlotEquiMap, var = climo * 30 * 24 * 3600 * 1000, c(2, 3), lon = lons3, lat = lats3, 
		   filled.continents = F, axelab = F, brks = seq(0, 250, 15),
		   titles = c("DJF", "MAM", "JJA", "SON"), toptitle = "prlr",  units = "mm/month", 
		   fileout = "climo_prlr.pdf", width = 10, height = 7)
	PlotLayout(PlotEquiMap, var = Corr, c(2, 3), lon = lons3, lat = lats3,
		   filled.continents = F, axelab = F, brks = seq(0.5, 1, 0.02), 
		   titles = c("DJF", "MAM", "JJA", "SON"), toptitle = "prlr", units = "Correlation", 
		   fileout = "corr_prlr.pdf", width = 10, height = 7)
	PlotLayout(PlotEquiMap, var = 100 * pc, c(2, 3), lon = lons3, lat = lats3,
		   filled.continents = F, axelab = F, brks=seq(50, 100, 2.5),
		   titles = c("DJF", "MAM", "JJA", "SON"), toptitle = "prlr", units = "% correct terciles",
		   fileout = "pc_prlr.pdf", width = 10, height = 7)
} else {
	PlotLayout(PlotEquiMap, var = climo -273.15, c(2, 3), lon = lons3, lat = lats3, 
		   filled.continents = F, axelab = F, brks = seq(-10, 35, 2),
		   titles = c("DJF", "MAM", "JJA", "SON"), toptitle = "tas",  units = "ºC", 
		   fileout = "climo_tas.pdf", width = 10, height = 7)
	PlotLayout(PlotEquiMap, var = Corr, c(2, 3), lon = lons3, lat = lats3,
		   filled.continents = F, axelab = F, brks = seq(0.5, 1, 0.02), 
		   titles = c("DJF", "MAM", "JJA", "SON"), toptitle = "tas", units = "Correlation", 
		   fileout = "corr_tas.pdf", width = 10, height = 7)
	PlotLayout(PlotEquiMap, var = 100 * pc, c(2, 3), lon = lons3, lat = lats3,
		   filled.continents = F, axelab = F, brks=seq(50, 100, 2.5),
		   titles = c("DJF", "MAM", "JJA", "SON"), toptitle = "tas", units = "% correct terciles",
		   fileout = "pc_tas.pdf", width = 10, height = 7)
}

